/***************************************************************
 *                 Mathematical Object Library                 *
 *                    simula+@metz.ensam.fr                    *
 *                   GNU/linux version 0.3.2                   *
 *            software under General Public License            *
 ***************************************************************
 * copyright  2002,2005,2006 COLLARD Christophe
 * copyright  2002,2005,2006 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554)
 * copyright  2002,2005,2006 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
 ***************************************************************/

/*!
    \brief mathematical tools library \n

    \htmlonly 
    <FONT color="#838383">

    This library belongs to Mathematical Object Libraries (MOL++) </br>
    MOL++ is part of Simula+ <br><br>

    Simula+ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version. <br><br>

    Simula+ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. <br><br>

    You should have received a copy of the GNU General Public License
    along with Simula+; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    </FONT>
    \endhtmlonly

    \author copyright \htmlonly &#169; \endhtmlonly 2002, 2005, 2006 Christophe COLLARD \n
	    copyright \htmlonly &#169; \endhtmlonly 2002, 2005, 2006 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
	    copyright \htmlonly &#169; \endhtmlonly 2002, 2005, 2006 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
    \version 0.3.2
    \date 2002-2006
    \bug none
    \warning none
*/

#ifndef __cplusplus
#error Must use C++ for the type maths
#endif

#if !defined(__MATHS__H)
#define __MATHS__H


#if !defined(__ASSERT_H)
#include <assert.h>
#endif

#if !defined(__PARAMETERS_H)
#include "../parameters.h"
#endif

#if !defined(__MATH_H)
#include <math.h>
#endif

using namespace std;


//-------------------------------
template <typename T> T abs (T x)
//-------------------------------
{
  if (x < (T) 0) return -x;
  else return x;
}


//-------------------------------------------
template <class T> T power (const T& x,int n)
//-------------------------------------------
{
  T value = 1;
  for (int i=1; i<=n; i++)
    value *= x;
  return value;
}
  

//------------------------------
template <class T> int sgn (T x)
//------------------------------
{ int sign = 1;
  if (x < 0) sign = -1;
  return sign;
}


//---------------
long double pi ()
//---------------
{ return 3.14159265358979323846264338327950288419716939937510;
}


//----------------------------------
long double factorial (const int& n)
//----------------------------------
{
  if (n>2) return n*(factorial(n-1));
  else return n;
}


//---------------------------------------------
template <class T> int inv_factorial (T number)
//---------------------------------------------
{
  assert (number >0);
  int i;

  for (i=0; number>1;)
    number /= ++i;
  if (abs(number-1)>epsilon) i=-1; // error !

  return i;
}


#endif
