/***************************************************************
 *                    simula+@metz.ensam.fr                    *
 *                   GNU/linux version 0.1.0                   *
 *            software under General Public License            *
 ***************************************************************
 * copyright  2005,2006 COLLARD Christophe
 * copyright  2005,2006 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554)
 * copyright  2005,2006 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
 ***************************************************************/

/*
    legendre-test belongs to Mathematical Object Libraries (MOL++)
    MOL++ is part of Simula+

    Simula+ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Simula+ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Simula+; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __cplusplus
#error Must use C++ for Legendre-test
#endif

#if !defined(__LEGENDRE_TEST_H)
#define _legendre_test_h


#if !defined(__IOSTREAM_H)
#include <iostream>
#endif

#if !defined(__STDIO_H)
#include <stdio.h>
#endif

#if !defined (__STDLIB_H)
#include <stdlib.h>
#endif

#if !defined(__VECTORS_H)
#include "../../MOL++/vectors.h"
#endif

#if !defined(__POLYNOMS_H)
#include "../../MOL++/polynoms.h"
#endif

#if !defined(__LEGENDRE_H)
#include "../../MOL++/legendre.h"
#endif

#if !defined(__AFFICHE_h)
#include "../affiche.h"
#endif


//============================
int test_legendre (int detail)
//============================
{
  int result=1;
  polynom<long double> P(4), Q(3), dP, dQ;
  Legendre<long double> L4(4), L3(3);

  P[4] = 35./8.;
  P[2] = -15./4;
  P[0] = 3./8.;

  Q[3] = 5./2.;
  Q[1] = -3./2.;

  dP = P.D_dx();
  dQ = Q.D_dx();

  bool test=true;
  for (long double i=-0.9; i<=0.9 && test; i+=0.1)
    { test *= ( abs(L4(i)- P(i))<epsilon && abs(L4.d_dx(i)-dP(i))<epsilon );
      test *= ( abs(L3(i)-Q(i))<epsilon && abs(L3.d_dx(i)-dQ(i))<epsilon );
    }
  if (detail) affiche ("operator () and d_dx()", test);
  else result *= test;

  cout << endl;  

  cout << "============================================================== \n";
  if (result) cout<< "                     Legendre test passed \n";
  else cout << "                     Legendre test failed \n";
  cout << "============================================================== \n";
  return result;
}



#endif
