/***************************************************************
 *                    simula+@metz.ensam.fr                    *
 *                   GNU/linux version 1.3.4                   *
 *            software under General Public License            *
 ***************************************************************
 * copyright  2003,2004,2005,2006 COLLARD Christophe
 * copyright  2003,2004,2005,2006 Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554)
 * copyright  2003,2004,2005,2006 Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
 ***************************************************************/

/*
    tensors3-test belongs to Mathematical Object Libraries (MOL++)
    MOL++ is part of Simula+

    Simula+ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Simula+ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Simula+; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __cplusplus
#error Must use C++ for tensor3-test
#endif

#if !defined(__TENSORS3_TEST_H)
#define _tensors3_test_h


#if !defined(__IOSTREAM_H)
#include <iostream>
#endif

#if !defined(__STDIO_H)
#include <stdio.h>
#endif

#if !defined (__STDLIB_H)
#include <stdlib.h>
#endif

#if !defined(__VECTORS_H)
#include "../../MOL++/vectors.h"
#endif

#if !defined(__MATRIX_H)
#include "../../MOL++/matrix.h"
#endif

#if !defined(__SYMMATRIX_H)
#include "../../MOL++/symmatrix.h"
#endif

#if !defined(__TENSORS2_H)
#include "../../MOL++/tensors2.h"
#endif

#if !defined(__TENSORS3_H)
#include "../../MOL++/tensors3.h"
#endif

#if !defined(__AFFICHE_h)
#include "../affiche.h"
#endif


//===========================
int test_tensor3 (int detail)
//===========================
{
  int result=1;

  tensor3<long double> t1(3,5,7), t2(3,5,7), t3, t4(3,5,7), t5(3,5,7);

  bool test = true;
  int size = 0;
  if (t1) size = t1;
  else test = false;
  if (t3) test = false;
  if (detail) affiche ("cast operator () tensor3 -> int", test && size==t1.dim1());
  else result *= (test && size==t1.dim1());

  if (detail) affiche ("operator !", !(!t1) && !t3);
  else result *= (!(!t1) && !t3);


  for (int i=1; i<=3; i++)
    for (int j=1; j<=5; j++)
      for (int k=1; k<=7; k++)
	{
	  t1(i,j,k) = i+j/(1.*k);
	  t4(i,j,k) = 2*i-5*j+7*k;
	}
  if (detail) affiche ("operator (,,)", t1(3,3,3)==4. && t1(1,5,4)==2.25);
  else result *= (t1(3,3,3)==4. && t1(1,5,4)==2.25);

  if (detail) affiche ("dim1 dim2 dim3", t1.dim1()==3 && t1.dim2()==5 && t1.dim3()==7);
  else result *= t1.dim1()==3 && t1.dim2()==5 && t1.dim3()==7;

  tensor3<long double> t6, t9;
  t6.assign(5,2,4);
  if (detail) affiche ("assign", t6.dim1()==5 && t6.dim2()==2 && t6.dim3()==4);
  else result *= t6.dim1()==5 && t6.dim2()==2 && t6.dim3()==4;

  t9.create(5);
  for (int i=1; i<=5; i++)
    for (int j=1; j<=2; j++)
      for (int k=1; k<=4; k++)
	t6 (i,j,k) = i - 2*j + 3*k;
  for (int i=1; i<=5; i++)
    t9[i] = t6[i]; 
  if (detail) affiche ("create", t9==t6);
  else result *= t9==t6;

  matrix<long double> matens (5,7);
  for (int i=1; i<=5; i++)
    for (int j=1; j<=7; j++)
      matens(i,j) = 2+i/(1.*j);
  int resultat = (t1[2]==matens);
  matens (1,1) = -1234;
  tensor3<long double> t1b(2,5,7);
  t1b(2,3,5) = 874/29.+47;
  if (detail) affiche ("operator []", resultat*(t1[2]!=matens) && (t1b[2][3][5]==t1b(2,3,5)));
  else result *= (resultat*(t1[2]!=matens) && (t1b[2][3][5]==t1b(2,3,5)));

  
  if (detail) affiche ("operator ==", (t1==t1) && !(t1==t4));
  else result *= ((t1==t1) && !(t1==t4));

  if (detail) affiche ("operator !=", !(t1!=t1) && t1!=t4);
  else result *= (!(t1!=t1) && t1!=t4);

  tensor3<long double> mtc(7,7,7), mtc2(7,7,7,true,-4.32), mtc3(7), mtc4(7,true,-4.32);
  for (int i=1; i<=7; i++)
    for (int j=1; j<=7; j++)
      for (int k=1; k<=7; k++)
	mtc(i,j,k) = mtc3(i,j,k) = -4.32;
  if (detail) affiche ("constructor", mtc==mtc2 && mtc==mtc3 && mtc==mtc4);
  else result *= mtc==mtc2 && mtc==mtc3 && mtc==mtc4;

  t2 = t1;
  resultat = (t1==t2);
  t1(1,1,1) = -10;
  resultat *= (t1!=t2);
  t3=t1;
  if (detail) affiche ("operator =", resultat*(t1==t3));
  else result *= (resultat*(t1==t3));

  tensor3<long double> ntsr1(9,5,7), ntsr2(9,5,7), ntsr3, ntsr4, ntsr0(9,5,7);
  for (int i=1; i<=9; i++)
    for (int j=1; j<=5; j++)
      for (int k=1; k<=7; k++)
	{ ntsr1(i,j,k) = 20*i/3. - 3*j/(1.*k);
	  ntsr2(i,j,k) = 3*i - j + 1/(1.*k);
	}
  ntsr3 = ntsr1 + ntsr2;
  ntsr4 = &(ntsr1+ntsr2);
  ntsr0 = &(ntsr1+ntsr2);
  tensor3<long double> ntsr5 = &(ntsr1+ntsr2);
  if (detail) affiche("operator = (destructive copy for temporary objects)", ntsr3==ntsr0 && ntsr3==ntsr4 && ntsr3==ntsr5);
  else result *= (ntsr3==ntsr0 && ntsr3==ntsr4 && ntsr3==ntsr5);

  tensor3<long double> mtrx, mtrx2(2,3,5), mtrx3(12,24,36), mtrx4, mtrx5(9,5,12), mtrx6(9,11,12), mtrx7, mtrx8;
  mtrx = mtrx8 = ntsr1;
  mtrx2 &= mtrx;
  mtrx3 &= mtrx4;
  mtrx5 &= ntsr1;
  mtrx6 &= ntsr1;
  mtrx7 &= mtrx8;
  if (detail) affiche ("operator &= (copy without size check)", mtrx2==ntsr1 && mtrx && !mtrx3 && mtrx7==ntsr1 && mtrx5==ntsr1 && mtrx6==ntsr1);
  else result *= (mtrx2==ntsr1 && mtrx && !mtrx3 && mtrx7==ntsr1 && mtrx5==ntsr1 && mtrx6==ntsr1);

  tensor3<long double> mtrx9, mtrx10;
  mtrx = mtrx8 = ntsr1;
  mtrx2 &=& mtrx;
  mtrx3 &=& (ntsr1+ntsr1);
  mtrx4 &=& mtrx;
  mtrx5  &=& mtrx10;
  mtrx9 &=& mtrx8;
  if (detail) affiche ("operator &= (destructive copy for temporary objects with no size check)", mtrx2==ntsr1 && !mtrx && mtrx3==(long double) 2*ntsr1 && !mtrx4 && !mtrx5 && mtrx9==ntsr1 && !mtrx8);
  else result *= (mtrx2==ntsr1 && !mtrx && mtrx3==(long double) 2*ntsr1 && !mtrx4 && !mtrx5 && mtrx9==ntsr1 && !mtrx8);

  tensor3<long double> t7(3,5,7);
  for (int i=1; i<=3; i++)
    for (int j=1; j<=5; j++)
      for (int k=1; k<=7; k++)
	{ t3(i,j,k) = i*0.5 + j*2.658 - k*3.12;
	  t7(i,j,k) = 4*t3(i,j,k);
	}
  if (detail) affiche ("operator +", t7==t3+t3+t3+t3);
  else result *= (t7==t3+t3+t3+t3);

  if (detail) affiche ("operator -", t3==t7-t3-t3-t3);
  else result *= (t3==t7-t3-t3-t3);

  t3 = t2;
  t3 += t1;
  if (detail) affiche ("operator +=", t3==t2+t1);
  else result *= (t3==t2+t1);
  
  t3-= t2;
  if (detail) affiche ("operator -=", t3==t1);
  else result *= (t3==t1);
 

  long double sum = 0;
  for (int i=1; i<=t1.dim1(); i++)
    for (int j=1; j<=t1.dim2(); j++)
      for (int k=1; k<=t1.dim3(); k++)
	sum += t1(i,j,k) * t4(i,j,k);
  if (detail) affiche ("operator |", (t1|t4)==sum);
  else result *= ((t1|t4)==sum);

  t3 = t2;
  t3 *= 5;
  if (detail) affiche ("operator *= (tensor/scalar)", t3==t2+t2+t2+t2+t2);
  else result *= (t3==t2+t2+t2+t2+t2);

  t3 = t2* (long double) 3;
  if (detail) affiche ("operator * (tensor/scalar)", t3==t2+t2+t2);
  else result *= (t3==t2+t2+t2);
  
  t3 = (long double) 5*t1;
  if (detail) affiche ("operator * (scalar/tensor)", t3==t1+t1+t1+t1+t1);
  else result *= (t3==t1+t1+t1+t1+t1);
  
  tensor3<long double> t8(2,3,4);
  long double value = 0.125;
  for (int i=1; i<=2; i++)
    for (int j=1; j<=3; j++)
      for (int k=1; k<=4; k++)
	t8(i,j,k) = (17*i+j*7-25/(1.*k))*100;
  vector<long double> vt64(4);
  for (int k=1; k<=4; k++) vt64[k] = value*k;
  tensor2<long double> mat6_23, matest_23(2,3);
  for (int i=1; i<=2; i++)
    for (int j=1; j<=3; j++)
      for (int k=1; k<=4; k++)
	matest_23(i,j) += t8(i,j,k)*vt64[k];
  mat6_23 = t8*vt64;
  if (detail) affiche ("operator * (tensor/vector : P_kij V_j)", mat6_23==matest_23);
  else result *= (mat6_23==matest_23);
  
  vector<long double> vt62(2);
  for (int k=1; k<=2; k++) vt62[k] = value*k;
  matrix<long double> mat6_34, matest_34(3,4);
  for (int i=1; i<=2; i++)
    for (int j=1; j<=3; j++)
      for (int k=1; k<=4; k++)
 	matest_34(j,k) += t8(i,j,k)*vt62[i];
  mat6_34 = vt62*t8;
  if (detail) affiche ("operator * (vector/tensor : P_kij V_k)", mat6_34==matest_34);
  else result *= (mat6_34==matest_34);
  
  vector<long double> vt63(3);
  for (int k=1; k<=3; k++) vt63[k] = value*k;
  matrix<long double> mat6_24, matest_24(2,4);
  for (int i=1; i<=2; i++)
    for (int j=1; j<=3; j++)
      for (int k=1; k<=4; k++)
	matest_24(i,k) += t8(i,j,k)*vt63[j];
  mat6_24 = t8|vt63;
  if (detail) affiche ("operator | (vector/tensor : P_kij V_i)", mat6_24==matest_24);
  else result *= (mat6_24==matest_24);

  if (detail) affiche ("operator | (tensor/vector : P_kij V_i)", (vt63|t8)==matest_24);
  else result *= ((vt63|t8)==matest_24);

  if (detail) affiche ("sum tensor * vector (i), i = 1->3 :", (tensor_sum(t8,vt62,1) == mat6_34)  &&  (tensor_sum(t8,vt63,2) == mat6_24)  &&  (tensor_sum(t8,vt64,3) == mat6_23) );
  else result *= ((tensor_sum(t8,vt62,1) == mat6_34)  &&  (tensor_sum(t8,vt63,2) == mat6_24)  &&  (tensor_sum(t8,vt64,3) == mat6_23) );

  tensor3<long double> ts3(5,7,9);
  tensor2<long double> ts2(7,9);
  vector<long double> resp(5);
  for (int i=1; i<=5; i++)
    for (int j=1; j<=7; j++)
      for (int k=1; k<=9; k++)
	{ ts3(i,j,k) = i + 5*j - 3*k;
	  ts2(j,k) += (5*j - 7*k/3.)*i;
	}
  for (int i=1; i<=5; i++)
    for (int j=1; j<=7; j++)
      for (int k=1; k<=9; k++)
	resp[i] += ts3(i,j,k) * ts2(j,k);
  if (detail) affiche ("operator || ( tensor || matrix : P_n,ij || M_ij)", resp == (ts3||ts2));
  else result *= (resp == (ts3||ts2));

  tensor3<long double> tsm(3,5,7), tresult(2,5,7);
  for (int i=1; i<=3; i++)
    for (int j=1; j<=5; j++)
      for (int k=1; k<=7; k++)
	tsm(i,j,k) = i/(1.*j) + k*j - i;
  for (int p=1; p<=2; p++)
    for (int i=1; i<=3; i++)
      for (int j=1; j<=5; j++)
	for (int k=1; k<=7; k++)
	  tresult(p,j,k) += matest_23(p,i) * tsm(i,j,k);
  if (detail) affiche ("operator * ( matrix * tensor : M_ij * P_jkl)", tresult==matest_23*tsm);
  else result *= (tresult==matest_23*tsm);

  ts3.save("tensor3.res");
  tensor3<long double> ncd(ts3.dim1(), ts3.dim2(), ts3.dim3());
  ncd.read("tensor3.res");
  if (detail) affiche("read/write", ncd==ts3);
  else result *= (ncd==ts3);

  cout << endl;  

  cout << "============================================================== \n";
  if (result) cout<< "               3rd order tensor test passed \n";
  else cout << "               3rd order tensor test failed \n";
  cout << "============================================================== \n";

  return result;
}


#endif
